// file: random_gen.cc
//
// random number generator

// include files
//
#include "random_gen.h"

// method: random_cc
//
// arguments: none
//
// return: a float_4 random number uniformly distributed in (0, 1)
//
// this function uses Knuth's subtractive algorithm to generate
// uniform random numbers in (0, 1) and then scales them to the
// desired limits
//
float_4 random_cc() {
  
  // define initialization variables
  //
  static int_4 rand_init = ISIP_RAND_INIT;
  static int_4 temp_init = (int_4)0;
  
  // define local variables
  //
  static int_4* m_array = new int_4 [ISIP_RAND_ARRAY];
  static int_4 next_i = (int_4)0;
  static int_4 next_ip = (int_4)0;
  
  // define temporary variables
  //
  int_4 mj = (int_4)0;
  int_4 mk = (int_4)0;
  int_4 ii = (int_4)0;
  int_4 i = (int_4)0;
  int_4 k = (int_4)0;
  
  // initialize generator
  //
  if ((rand_init < 0) || (temp_init == 0)) {

    // set values
    //
    temp_init = (int_4)1;
    mj = ISIP_RAND_SEED - (rand_init < 0 ? -rand_init : rand_init);
    mj %= ISIP_RAND_BIG;
    m_array[ISIP_RAND_ARRAY - 1] = mj;
    mk = (int_4)1;
    
    // now initialize rest of array
    //
    ii = (int_4)0;
    for (i = 0; i < ISIP_RAND_ARRAY - 1; i++) {
      ii = (21 * i) % (ISIP_RAND_ARRAY - 1);
      m_array[ii] = mk;
      mk = mj - mk;
      if (mk < 0) {
	mk += ISIP_RAND_BIG;
      }
      mj = m_array[ii];
    }
    
    // randomize these numbers
    //
    for (k = 0; k < 5; k ++) {
      for (i = 0; i < ISIP_RAND_ARRAY; i++) {
	m_array[i] -= m_array[1 + (i + ISIP_RAND_CONST - 1) % 
				 (ISIP_RAND_ARRAY - 1)];
	if (m_array[i] < 0) {
	  m_array[i] += ISIP_RAND_BIG;
	}
      }
    }
    
    // set values for next round
    //
    next_i = (int_4)0;
    next_ip = ISIP_RAND_CONST;
    rand_init = (int_4)1;
  }
  
  // the actual generator --- this is what is called every time except
  // the first (initialization) time
  //
  if (++next_i == ISIP_RAND_ARRAY) {
    next_i = (int_4)1;
  }
  if (++next_ip == ISIP_RAND_ARRAY) {
    next_ip = (int_4)1;
  }
  mj = m_array[next_i] - m_array[next_ip];
  if (mj < 0) {
    mj += ISIP_RAND_BIG;
  }
  m_array[next_i] = mj;
  
  // scale the number between (0, 1)
  //
  float_4 randnum = (float_4)mj * ISIP_RAND_SCALE;
  
  // return the random number
  //
  return (randnum);
}

// method: random_cc
//
// arguments: none
//  float_4 low: (input) the lower bound on random values
//  float_4 high: (input) the higher bound on random values
//
// return: a float_4 random number uniformly distributed in the
//         desired limits
//
// this function uses Knuth's subtractive algorithm to generate
// uniform random numbers in (0, 1) and then scales them to the
// desired limits
//
float_4 random_cc(float_4 low_a, float_4 high_a) {
  
  // exit gracefully by creating a random number in (0, 1) and scaling
  // it and returning the final value
  //
  return (high_a - low_a) * random_cc() + low_a;
}

// method: random_cc
//
// arguments: none
//  float_4 limit: (input) the range on random values
//
// return: a float_4 random number uniformly distributed in the
//         desired limit
//
// this function uses Knuth's subtractive algorithm to generate
// uniform random numbers in (0, 1) and then scales them to the
// desired limit
//
float_4 random_cc(float_4 limit_a) {

  // make sure the limit is positive
  //
  float_4 poslim = fabs(limit_a);
  
  // exit gracefully by creating a random number in (0, 1) and scaling
  // it and returning the final value
  //
  return random_cc(-poslim, poslim);
}
